import { Directive, ElementRef, HostListener, Inject, NgModule, Renderer2 } from '@angular/core';
import { polyfill } from 'smoothscroll-polyfill';
import { DOCUMENT } from '@angular/platform-browser';
import { extractElementPosition } from 'ng-html-util';

var ScrollSpyDirective = (function () {
    /**
     * @param {?} document
     * @param {?} el
     * @param {?} renderer
     */
    function ScrollSpyDirective(document, el, renderer) {
        this.document = document;
        this.el = el;
        this.renderer = renderer;
        this.elements = [];
        this.directNavigation = false;
    }
    /**
     * @return {?}
     */
    ScrollSpyDirective.prototype.ngAfterContentInit = function () {
        this.collectIds();
    };
    /**
     * @return {?}
     */
    ScrollSpyDirective.prototype.collectIds = function () {
        this.elements = [];
        var /** @type {?} */ elements = this.el.nativeElement.querySelectorAll('a');
        for (var /** @type {?} */ i = 0; i < elements.length; i++) {
            var /** @type {?} */ elem = elements.item(i);
            var /** @type {?} */ id = ScrollSpyDirective.getId(elem);
            if (!id)
                continue;
            var /** @type {?} */ destination = this._getPeerElement(id);
            if (!destination)
                continue;
            elem.addEventListener('click', this._onLinkClicked.bind(this));
            this.elements.push({
                id: id,
                link: elem,
                destination: destination
            });
        }
    };
    /**
     * @param {?} event
     * @return {?}
     */
    ScrollSpyDirective.prototype._onLinkClicked = function (event) {
        event.preventDefault();
        var /** @type {?} */ target = event.currentTarget;
        var /** @type {?} */ id = ScrollSpyDirective.getId(target);
        var /** @type {?} */ destination = this._getPeerElement(id);
        this.directNavigation = true;
        var /** @type {?} */ position = extractElementPosition(this.document, destination);
        window.scrollTo({ top: position.top - 25, left: 0, behavior: 'smooth' });
        this._cleanCurrentLink();
        this._setCurrentLink(target);
        this.directNavigation = false;
    };
    /**
     * @param {?} id
     * @return {?}
     */
    ScrollSpyDirective.prototype._getPeerElement = function (id) {
        var /** @type {?} */ destination = this.document.getElementById(id);
        if (!destination)
            return null;
        return destination;
    };
    /**
     * @param {?} elem
     * @return {?}
     */
    ScrollSpyDirective.getId = function (elem) {
        var /** @type {?} */ href = elem.getAttribute('href');
        if (!href)
            return null;
        return href.replace('#', '');
    };
    /**
     * @param {?} event
     * @return {?}
     */
    ScrollSpyDirective.prototype.onWindowScroll = function (event) {
        if (this.directNavigation)
            return;
        for (var _i = 0, _a = this.elements; _i < _a.length; _i++) {
            var elem = _a[_i];
            var /** @type {?} */ top_1 = elem.destination.getBoundingClientRect().top;
            if (top_1 > 0 && top_1 < 25) {
                this._cleanCurrentLink();
                this._setCurrentLink(elem.link);
                break;
            }
        }
    };
    /**
     * @return {?}
     */
    ScrollSpyDirective.prototype._cleanCurrentLink = function () {
        if (!this.currentActiveLink)
            return;
        this.renderer.removeClass(this.currentActiveLink, 'active');
    };
    /**
     * @param {?} elem
     * @return {?}
     */
    ScrollSpyDirective.prototype._setCurrentLink = function (elem) {
        this.currentActiveLink = elem;
        this.renderer.addClass(this.currentActiveLink, 'active');
    };
    return ScrollSpyDirective;
}());
ScrollSpyDirective.decorators = [
    { type: Directive, args: [{
                selector: '[scroll-spy]'
            },] },
];
/**
 * @nocollapse
 */
ScrollSpyDirective.ctorParameters = function () { return [
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] },] },
    { type: ElementRef, },
    { type: Renderer2, },
]; };
ScrollSpyDirective.propDecorators = {
    'onWindowScroll': [{ type: HostListener, args: ["window:scroll", ['$event'],] },],
};

var ScrollSpyModule = (function () {
    function ScrollSpyModule() {
        polyfill();
    }
    return ScrollSpyModule;
}());
ScrollSpyModule.decorators = [
    { type: NgModule, args: [{
                declarations: [ScrollSpyDirective],
                exports: [ScrollSpyDirective]
            },] },
];
/**
 * @nocollapse
 */
ScrollSpyModule.ctorParameters = function () { return []; };

/**
 * Generated bundle index. Do not edit.
 */

export { ScrollSpyModule, ScrollSpyDirective };
