import { Directive, ElementRef, HostListener, Inject, NgModule, Renderer2 } from '@angular/core';
import { polyfill } from 'smoothscroll-polyfill';
import { DOCUMENT } from '@angular/platform-browser';
import { extractElementPosition } from 'ng-html-util';

class ScrollSpyDirective {
    /**
     * @param {?} document
     * @param {?} el
     * @param {?} renderer
     */
    constructor(document, el, renderer) {
        this.document = document;
        this.el = el;
        this.renderer = renderer;
        this.elements = [];
        this.directNavigation = false;
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        this.collectIds();
    }
    /**
     * @return {?}
     */
    collectIds() {
        this.elements = [];
        let /** @type {?} */ elements = this.el.nativeElement.querySelectorAll('a');
        for (let /** @type {?} */ i = 0; i < elements.length; i++) {
            let /** @type {?} */ elem = elements.item(i);
            let /** @type {?} */ id = ScrollSpyDirective.getId(elem);
            if (!id)
                continue;
            let /** @type {?} */ destination = this._getPeerElement(id);
            if (!destination)
                continue;
            elem.addEventListener('click', this._onLinkClicked.bind(this));
            this.elements.push({
                id,
                link: elem,
                destination
            });
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _onLinkClicked(event) {
        event.preventDefault();
        let /** @type {?} */ target = event.currentTarget;
        let /** @type {?} */ id = ScrollSpyDirective.getId(target);
        let /** @type {?} */ destination = this._getPeerElement(id);
        this.directNavigation = true;
        let /** @type {?} */ position = extractElementPosition(this.document, destination);
        window.scrollTo({ top: position.top - 25, left: 0, behavior: 'smooth' });
        this._cleanCurrentLink();
        this._setCurrentLink(target);
        this.directNavigation = false;
    }
    /**
     * @param {?} id
     * @return {?}
     */
    _getPeerElement(id) {
        let /** @type {?} */ destination = this.document.getElementById(id);
        if (!destination)
            return null;
        return destination;
    }
    /**
     * @param {?} elem
     * @return {?}
     */
    static getId(elem) {
        let /** @type {?} */ href = elem.getAttribute('href');
        if (!href)
            return null;
        return href.replace('#', '');
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onWindowScroll(event) {
        if (this.directNavigation)
            return;
        for (let /** @type {?} */ elem of this.elements) {
            let /** @type {?} */ top = elem.destination.getBoundingClientRect().top;
            if (top > 0 && top < 25) {
                this._cleanCurrentLink();
                this._setCurrentLink(elem.link);
                break;
            }
        }
    }
    /**
     * @return {?}
     */
    _cleanCurrentLink() {
        if (!this.currentActiveLink)
            return;
        this.renderer.removeClass(this.currentActiveLink, 'active');
    }
    /**
     * @param {?} elem
     * @return {?}
     */
    _setCurrentLink(elem) {
        this.currentActiveLink = elem;
        this.renderer.addClass(this.currentActiveLink, 'active');
    }
}
ScrollSpyDirective.decorators = [
    { type: Directive, args: [{
                selector: '[scroll-spy]'
            },] },
];
/**
 * @nocollapse
 */
ScrollSpyDirective.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] },] },
    { type: ElementRef, },
    { type: Renderer2, },
];
ScrollSpyDirective.propDecorators = {
    'onWindowScroll': [{ type: HostListener, args: ["window:scroll", ['$event'],] },],
};

class ScrollSpyModule {
    constructor() {
        polyfill();
    }
}
ScrollSpyModule.decorators = [
    { type: NgModule, args: [{
                declarations: [ScrollSpyDirective],
                exports: [ScrollSpyDirective]
            },] },
];
/**
 * @nocollapse
 */
ScrollSpyModule.ctorParameters = () => [];

/**
 * Generated bundle index. Do not edit.
 */

export { ScrollSpyModule, ScrollSpyDirective };
